unit Unit1;

{$mode objfpc}{$H+}

{$DEFINE USE_MESSAGE_PUMP}

interface

uses
  Classes, SysUtils,
  Forms, Controls, Graphics, Dialogs, StdCtrls, ExtCtrls,
  fpwebsocketclient, fpwebsocket, opensslsockets;
type

  { TForm1 }

  TForm1 = class(TForm)
    Edit1: TEdit;
    IdleTimer1: TIdleTimer;
    SendButton: TButton;
    CheckBox1: TCheckBox;
    ComboBox1: TComboBox;
    Label1: TLabel;
    Memo1: TMemo;
    Panel1: TPanel;
    procedure CheckBox1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure IdleTimer1Timer(Sender: TObject);
    procedure SendButtonClick(Sender: TObject);
  private
    FPump: TWSThreadMessagePump;
    FClient: TWebsocketClient;
    procedure DoConnect(Sender: TObject);
    procedure DoControl(Sender: TObject; aType: TFrameType; const aData: TBytes);
    procedure AddLine(ALine: String);
    procedure DoDisconnect(Sender: TObject);
    procedure DoHandshakeResponse(Sender: TObject;
      aResponse: TWSHandShakeResponse; var aAllow: Boolean);
    procedure DoMessage(Sender: TObject; const aMessage: TWSMessage);
    procedure DoSendHandshake(Sender: TObject; aHeaders: TStrings);

  public

  end;

var
  Form1: TForm1;

implementation
uses
  URIParser;

{$R *.lfm}

{ TForm1 }


procedure TForm1.CheckBox1Click(Sender: TObject);
var
  lURI: TURI;
begin
  if (not FClient.Active) and CheckBox1.Checked then
  begin
    lURI := ParseURI(ComboBox1.Text);

    if lURI.Port = 0 then
    begin

      if lURI.Protocol = 'ws' then lURI.Port:=80
      else if lURI.Protocol = 'wss' then lURI.Port:=443
      else
      begin
        AddLine(' - unvalid port for ' + ComboBox1.Text);
        CheckBox1.Checked:=False;
        Exit;
      end
    end;
    FClient.HostName:=lURI.Host;
    FClient.Port:=lURI.Port;
    FClient.Resource:=lURI.Document;
    FCLient.UseSSL:=lURI.Protocol='wss';
    FClient.Connect;
  end;
  if (FClient.Active) and not CheckBox1.Checked then
  begin
    FClient.Active:=False;
  end;
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
  FPump := TWSThreadMessagePump.Create(Self);
  FPump.Interval:=100;
  FPump.Execute;
  FClient := TWebsocketClient.Create(Self);
  FClient.Options:=[woIndividualFrames];
  FClient.OnConnect:=@DoConnect;
  FClient.OnControl:=@DoControl;
  FClient.OnDisconnect:=@DoDisconnect;
  FCLient.OnMessageReceived:=@DoMessage;
  FClient.OnSendHandShake:=@DoSendHandshake;
  FClient.OnHandshakeResponse:=@DoHandshakeResponse;
  FClient.OutGoingFrameMask:=RaNDOM(MaxInt);
  {$IFDEF USE_MESSAGE_PUMP}
  FClient.MessagePump := FPump;
  {$ENDIF}
end;

procedure TForm1.IdleTimer1Timer(Sender: TObject);
begin
  {$IfNDef USE_MESSAGE_PUMP}
  if FClient.Active then
    FClient.CheckIncoming;
  {$ENDIF}
end;

procedure TForm1.SendButtonClick(Sender: TObject);
begin
  if not FClient.Active then
   Exit;
  AddLine('>>> '+Edit1.Text);
  FClient.SendMessage(Edit1.Text);
  Edit1.Text:='';
end;

procedure TForm1.DoConnect(Sender: TObject);
begin
  Memo1.Lines.Add(' - Websocket connected -');
  SendButton.Enabled:=True;
end;

procedure TForm1.DoControl(Sender: TObject; aType: TFrameType;
  const aData: TBytes);
begin
  case aType of
    //ftContinuation,
    //ftText,
    ftBinary: AddLine(' - binary message received -');
    ftClose: AddLine(' - close message received -');
    ftPing:
      begin
        AddLine(' - ping recieved - '  );
        if Length(aData) > 0 then
          AddLine(Copy(PChar(@aData[0]), 0, Length(aData)));
        //FClient.Pong('');
      end;
    ftPong: AddLine(' - pong message recieved -');
    //ftFutureOpcodes

  end;
end;

procedure TForm1.AddLine(ALine: String);
begin
  Memo1.Lines.Add(ALine);
  Memo1.SelStart:=Length(Memo1.Text);
end;

procedure TForm1.DoDisconnect(Sender: TObject);
begin
  CheckBox1.Checked:=False;
  AddLine(' - Disconnected -');
  SendButton.Enabled:=False;
  //FClient.Active:=False;
end;

procedure TForm1.DoHandshakeResponse(Sender: TObject;
  aResponse: TWSHandShakeResponse; var aAllow: Boolean);
begin
  AddLine(aResponse.StatusText);
  ;
end;

procedure TForm1.DoMessage(Sender: TObject; const aMessage: TWSMessage);
begin
  if aMessage.IsText then
    AddLine('<<< '+aMessage.AsString)
  else
    AddLine('<<< - non text message received -');
end;

procedure TForm1.DoSendHandshake(Sender: TObject; aHeaders: TStrings);
begin
  Memo1.Lines.Assign(aHeaders);
end;

end.


